<?php

declare(strict_types=1);

namespace Drupal\slim_select\Plugin\Field\FieldWidget;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldWidget\OptionsSelectWidget;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Exception\ParseException;
use Symfony\Component\Yaml\Yaml;

/**
 * Plugin implementation of the 'slim_select' widget.
 */
#[FieldWidget(
  id: 'slim_select',
  label: new TranslatableMarkup('Slim Select'),
  field_types: [
    'entity_reference',
    'list_integer',
    'list_float',
    'list_string',
  ],
  multiple_values: TRUE,
)]
class SlimSelectWidget extends OptionsSelectWidget {

  public function __construct(
    string $plugin_id,
    mixed $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    array $third_party_settings,
    protected readonly ConfigFactoryInterface $configFactory,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): static {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    return ['settings' => []] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form = parent::settingsForm($form, $form_state);
    $settings = $this->getSetting('settings');
    $form['settings'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Slim Select settings'),
      '#description' => $this->t('A <b>YAML</b> formatted key-value array which will be passed as `#slim_select` element property.'),
      '#default_value' => $settings ? Yaml::dump($settings) : NULL,
      '#element_validate' => [[static::class, 'elementSettingsValidate']],
    ];
    return $form;
  }

  /**
   * Form element validation handler for the 'settings' element.
   */
  public static function elementSettingsValidate(&$element, FormStateInterface $form_state): void {
    try {
      $parsed = Yaml::parse($element['#value']);
    }
    catch (ParseException $exception) {
      $form_state->setError($element, t('Could not parse YAML settings: %exception.', ['%exception' => $exception->getMessage()]));
      return;
    }

    $form_state->setValueForElement($element, $parsed ?? []);
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = [];
    $settings = $this->getSetting('settings');
    if (!empty($settings)) {
      $summary[] = $this->t('Settings: @value', ['@value' => Yaml::dump($settings)]);
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);

    $settings = $this->getSetting('settings') ?? [];

    // Pass the empty option as placeholder if present.
    /* @see \Drupal\Core\Field\Plugin\Field\FieldWidget\OptionsSelectWidget::getEmptyLabel */
    if (isset($element['#options']['_none'])) {
      $version = $this->configFactory->get('slim_select.settings')->get('version');
      $placeholder_setting = \version_compare(\ltrim($version, 'v'), '2.0.0', '>=') ? 'placeholderText' : 'placeholder';
      $settings += [$placeholder_setting => $element['#options']['_none']];
      unset($element['#options']['_none']);
    }

    $element['#slim_select'] = $settings;
    return $element;
  }

}
