<?php

declare(strict_types=1);

namespace Drupal\Tests\slim_select\FunctionalJavascript;

use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the slim_select field widget.
 *
 * @group slim_select
 */
class SlimSelectWidgetTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'field',
    'field_ui',
    'options',
    'slim_select',
  ];

  /**
   * A user with admin permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->adminUser = $this->drupalCreateUser([], NULL, TRUE);
    $this->drupalLogin($this->adminUser);

    $this->createContentType(['type' => 'article', 'name' => 'Article']);

    // Create items for entity_reference field type.
    for ($i = 1; $i <= 3; $i++) {
      $this->drupalCreateNode(['type' => 'article', 'title' => "Item $i"]);
    }
  }

  /**
   * Data provider for widget field types.
   *
   * @return array
   *   Array of test parameters.
   */
  public static function providerSlimSelectWidget(): array {
    return [
      'list_string' => [
        'field_type' => 'list_string',
        'field_storage_settings' => ['allowed_values' => ['item1' => 'Item 1', 'item2' => 'Item 2']],
      ],
      'list_float' => [
        'field_type' => 'list_float',
        'field_storage_settings' => ['allowed_values' => ['1.0' => 'Item 1', '2.0' => 'Item 2']],
      ],
      'list_integer' => [
        'field_type' => 'list_integer',
        'field_storage_settings' => ['allowed_values' => ['1' => 'Item 1', '2' => 'Item 2']],
      ],
      'entity_reference' => [
        'field_type' => 'entity_reference',
        'field_storage_settings' => ['target_type' => 'node'],
      ],
    ];
  }

  /**
   * Tests the Slim Select widget with various field types.
   *
   * @dataProvider providerSlimSelectWidget
   */
  public function testSlimSelectWidget(string $field_type, array $field_storage_settings): void {
    $field_name = 'test_slim_select';
    $this->createSelectField(
      $field_name,
      $field_type,
      FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED,
      $field_storage_settings
    );

    $assert = $this->assertSession();
    $this->drupalGet('/node/add/article');
    $assert->elementExists('css', 'div.ss-main', $assert->fieldExists('edit-test-slim-select')->getParent());
  }

  /**
   * Create a select field on the article content type.
   */
  private function createSelectField(string $field_name, string $field_type, int $cardinality, array $field_storage_settings): void {
    $entity_type_manager = \Drupal::entityTypeManager();
    $entity_type_manager->getStorage('field_storage_config')->create([
      'field_name' => $field_name,
      'entity_type' => 'node',
      'type' => $field_type,
      'cardinality' => $cardinality,
      'settings' => $field_storage_settings,
    ])->save();
    $entity_type_manager->getStorage('field_config')->create([
      'label' => $field_name,
      'field_name' => $field_name,
      'entity_type' => 'node',
      'bundle' => 'article',
      'settings' => [],
    ])->save();

    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $display_repository */
    $display_repository = \Drupal::service('entity_display.repository');
    $display_repository->getFormDisplay('node', 'article')->setComponent($field_name, ['type' => 'slim_select'])->save();
    $display_repository->getViewDisplay('node', 'article')->setComponent($field_name, ['type' => 'list_default'])->save();
  }

}
